@@ -4,8 +4,9 @@
 import math
 import numpy as np
 from copy import deepcopy
-from sklearn.base import BaseEstimator, ClassifierMixin, RegressorMixin
-from sklearn.utils.multiclass import check_classification_targets
+
+from sklearn.base import BaseEstimator, ClassifierMixin
+from sklearn.utils.multiclass import check_classification_targets, unique_labels
 from sklearn.utils.validation import check_X_y, check_array, check_is_fitted
 
 from imodels.rule_list.rule_list import RuleList
@@ -110,6 +111,7 @@ class GreedyRuleListClassifier(BaseEstimator, RuleList, ClassifierMixin):
             self.depth += 1  # increase the depth since we call fit once
             self.rules_ = par_node
             self.complexity_ = len(self.rules_)
+            self.classes_ = unique_labels(y)
             return par_node
 
     def predict_proba(self, X):
@@ -208,8 +210,7 @@ class GreedyRuleListClassifier(BaseEstimator, RuleList, ClassifierMixin):
         cutoff = 0.5
 
         # iterate through each value in the column
-        for value in set(col):
-
+        for value in np.unique(col):
             # separate y into 2 groups
             y_predict = col < value
 
@@ -226,7 +227,7 @@ class GreedyRuleListClassifier(BaseEstimator, RuleList, ClassifierMixin):
         """Returns criterion calculated over a split
         split decision, True/False, and y_true can be multi class
         """
-        if len(split_decision) != len(y_real):
+        if split_decision.shape[0] != y_real.shape[0]:
             print('They have to be the same length')
             return None
 
@@ -245,16 +246,20 @@ class GreedyRuleListClassifier(BaseEstimator, RuleList, ClassifierMixin):
         s_right = criterion_func(y_real[~split_decision])
 
         # overall criterion, again weighted average
-        n = len(y_real)
-        sample_weights = np.ones(n)
+        n = y_real.shape[0]
         if self.class_weight is not None:
+            sample_weights = np.ones(n)
             for c in self.class_weight.keys():
                 idxs_c = y_real == c
                 sample_weights[idxs_c] = self.class_weight[c]
-        tot_weight = np.sum(sample_weights)
-        weight_left = np.sum(sample_weights[split_decision]) / tot_weight
-        weight_right = np.sum(sample_weights[~split_decision]) / tot_weight
-        s = weight_left * s_left + weight_right * s_right
+            total_weight = np.sum(sample_weights)
+            weight_left = np.sum(sample_weights[split_decision]) / total_weight
+            # weight_right = np.sum(sample_weights[~split_decision]) / total_weight
+        else:
+            tot_left_samples = np.sum(split_decision == 1)
+            weight_left = tot_left_samples / n
+
+        s = weight_left * s_left + (1 - weight_left) * s_right
         return s
 
     def gini_criterion(self, y):
@@ -262,13 +267,13 @@ class GreedyRuleListClassifier(BaseEstimator, RuleList, ClassifierMixin):
         = sum(pc * (1 – pc))
         '''
         s = 0
-        n = len(y)
-        classes = set(y)
+        n = y.shape[0]
+        classes = np.unique(y)
 
         # for each class, get entropy
         for c in classes:
             # weights for each class
-            n_c = sum(y == c)
+            n_c = np.sum(y == c)
             p_c = n_c / n
 
             # weighted avg
