@@ -2,37 +2,40 @@ from typing import Iterable, Tuple, List
 
 import numpy as np
 import pandas as pd
-from sklearn.ensemble import BaggingClassifier, BaggingRegressor, GradientBoostingRegressor, RandomForestRegressor
-from sklearn.tree import DecisionTreeClassifier, DecisionTreeRegressor
-from sklearn.utils.validation import check_X_y, check_array, check_is_fitted
 from mlxtend.frequent_patterns import fpgrowth
+from sklearn.ensemble import BaggingRegressor, GradientBoostingRegressor, RandomForestRegressor
+from sklearn.tree import DecisionTreeRegressor
+from sklearn.utils.validation import check_array
 
 from imodels.util.convert import tree_to_rules
-from imodels.util.discretization.mdlp import BRLDiscretizer
+from imodels.discretization import BRLDiscretizer, SimpleDiscretizer
 
 
 def extract_fpgrowth(X, y,
-                     feature_labels=None,
+                     feature_names,
                      minsupport=0.1,
                      maxcardinality=2,
                      undiscretized_features=[],
+                     disc_strategy='simple',
+                     disc_kwargs={},
                      verbose=False) -> Tuple[List[Tuple], BRLDiscretizer]:
-
     # deal with pandas data
     if type(X) in [pd.DataFrame, pd.Series]:
-        if feature_labels is None:
-            feature_labels = X.columns
+        if feature_names is None:
+            feature_names = X.columns
         X = X.values
     if type(y) in [pd.DataFrame, pd.Series]:
         y = y.values
 
-    if feature_labels is None:
-        feature_labels = [f'feature_{i}' for i in range(X.shape[1])]
-    
-    discretizer = BRLDiscretizer(X, y, feature_labels=feature_labels, verbose=verbose)
-    X = discretizer.discretize_mixed_data(X, y, undiscretized_features)
-    X_df_onehot = discretizer.onehot_df
-    
+    if disc_strategy == 'mdlp':
+        discretizer = BRLDiscretizer(X, y, feature_labels=feature_names, verbose=verbose)
+        discretizer.fit(X, y, undiscretized_features)
+    else:
+        discretizer = SimpleDiscretizer(**disc_kwargs)
+        discretizer.fit(X, feature_names)
+
+    X_df_onehot = discretizer.transform(X)
+
     # Now find frequent itemsets
     itemsets_df = fpgrowth(X_df_onehot, min_support=minsupport, max_len=maxcardinality)
     itemsets_indices = [tuple(s[1]) for s in itemsets_df.values]
@@ -45,23 +48,21 @@ def extract_fpgrowth(X, y,
 
 
 def extract_rulefit(X, y, feature_names,
+                    n_estimators=10,
                     tree_size=4,
-                    max_rules=2000,
                     memory_par=0.01,
                     tree_generator=None,
                     exp_rand_tree_size=True,
                     random_state=None) -> List[str]:
-
     if tree_generator is None:
-        n_estimators_default = int(np.ceil(max_rules / tree_size))
         sample_fract_ = min(0.5, (100 + 6 * np.sqrt(X.shape[0])) / X.shape[0])
 
-        tree_generator = GradientBoostingRegressor(n_estimators=n_estimators_default,
-                                                    max_leaf_nodes=tree_size,
-                                                    learning_rate=memory_par,
-                                                    subsample=sample_fract_,
-                                                    random_state=random_state,
-                                                    max_depth=100)
+        tree_generator = GradientBoostingRegressor(n_estimators=n_estimators,
+                                                   max_leaf_nodes=tree_size,
+                                                   learning_rate=memory_par,
+                                                   subsample=sample_fract_,
+                                                   random_state=random_state,
+                                                   max_depth=100)
 
     if type(tree_generator) not in [GradientBoostingRegressor, RandomForestRegressor]:
         raise ValueError("RuleFit only works with RandomForest and BoostingRegressor")
@@ -71,8 +72,7 @@ def extract_rulefit(X, y, feature_names,
         tree_generator.fit(X, y)
     else:  # randomise tree size as per Friedman 2005 Sec 3.3
         np.random.seed(random_state)
-        tree_sizes = np.random.exponential(scale=tree_size - 2,
-                                            size=int(np.ceil(max_rules * 2 / tree_size)))
+        tree_sizes = np.random.exponential(scale=tree_size - 2, size=n_estimators)
         tree_sizes = np.asarray([2 + np.floor(tree_sizes[i_]) for i_ in np.arange(len(tree_sizes))], dtype=int)
         tree_generator.set_params(warm_start=True)
         curr_est_ = 0
@@ -93,40 +93,38 @@ def extract_rulefit(X, y, feature_names,
         estimators_ = tree_generator.estimators_
 
     seen_antecedents = set()
-    extracted_rules = [] 
+    extracted_rules = []
     for estimator in estimators_:
         for rule_value_pair in tree_to_rules(estimator[0], np.array(feature_names), prediction_values=True):
             if rule_value_pair[0] not in seen_antecedents:
                 extracted_rules.append(rule_value_pair)
                 seen_antecedents.add(rule_value_pair[0])
-    
+
     extracted_rules = sorted(extracted_rules, key=lambda x: x[1])
     extracted_rules = list(map(lambda x: x[0], extracted_rules))
     return extracted_rules
 
 
-def extract_skope(X, y, feature_names, 
+def extract_skope(X, y, feature_names,
                   sample_weight=None,
                   n_estimators=10,
                   max_samples=.8,
                   max_samples_features=1.,
                   bootstrap=False,
                   bootstrap_features=False,
-                  max_depths=[3], 
-                  max_depth_duplication=None,
+                  max_depths=[3],
                   max_features=1.,
                   min_samples_split=2,
                   n_jobs=1,
                   random_state=None,
                   verbose=0) -> Tuple[List[str], List[np.array], List[np.array]]:
-    
     ensembles = []
     if not isinstance(max_depths, Iterable):
         max_depths = [max_depths]
 
     for max_depth in max_depths:
         bagging_clf = BaggingRegressor(
-            base_estimator= DecisionTreeRegressor(
+            base_estimator=DecisionTreeRegressor(
                 max_depth=max_depth,
                 max_features=max_features,
                 min_samples_split=min_samples_split
@@ -171,5 +169,5 @@ def extract_skope(X, y, feature_names,
     extracted_rules = []
     for estimator, features in zip(estimators_, estimators_features_):
         extracted_rules.append(tree_to_rules(estimator, np.array(feature_names)[features]))
-    
+
     return extracted_rules, estimators_samples_, estimators_features_
